@@ -97,6 +97,30 @@ static gint session_save_yourself_cb (GnomeClient *client, gint phase,
 				      gboolean fast,
 				      gpointer client_data);
 static void session_die_cb (GnomeClient *client, gpointer client_data);
+#else
+static GOptionEntry options[] = {
+    {
+        "write-source", 'w', 0, G_OPTION_ARG_NONE, &arg_write_source,
+        N_("Write the source code and exit"), NULL
+    },
+    {
+        "hide-palette", '\0', 0, G_OPTION_ARG_NONE, &arg_hide_palette,
+        N_("Start with the palette hidden"), NULL
+    },
+    {
+        "hide-property-editor", '\0', 0, G_OPTION_ARG_NONE, &arg_hide_property_editor,
+        N_("Start with the property editor hidden"), NULL
+    },
+    {
+        "show-widget-tree", '\0', 0, G_OPTION_ARG_NONE, &arg_show_widget_tree,
+        N_("Show the widget tree"), NULL
+    },
+    {
+        "show-clipboard", '\0', 0, G_OPTION_ARG_NONE, &arg_show_clipboard,
+        N_("Show the clipboard"), NULL
+    },
+    { NULL }
+};
 #endif
 
 
@@ -186,7 +210,7 @@ main (int argc, char *argv[])
       gtk_rc_add_default_file (rc_path);
       g_free (rc_path);
     }
-  gtk_init (&argc, &argv);
+  gtk_init_with_args (&argc, &argv, "file", options, NULL, NULL);
 #endif
 
   /* Ignore Ctrl-C. */
