@@ -130,7 +130,8 @@ static GladeError* source_backup_file_if_exists (const gchar * filename);
 
 static gchar * source_make_string_internal (const gchar * text,
 					    gboolean translatable,
-					    gboolean is_static);
+					    gboolean is_static,
+					    gboolean context);
 static void source_reset_code_buffers (GbWidgetWriteSourceData * data);
 static void source_destroy_standard_widgets_callback (gchar * key,
 						      GtkWidget * widget,
@@ -1901,7 +1902,26 @@ source_write_support_files (GbWidgetWriteSourceData * data)
   source_write_preamble (data->project_name, fp);
 
   if (glade_project_get_gnome_support (data->project))
-    fprintf (fp, "#include <gnome.h>\n\n");
+    {
+      fprintf (fp,
+	       "#ifdef HAVE_CONFIG_H\n"
+	       "#  include <config.h>\n"
+	       "#endif\n"
+	       "\n"
+	       "#include <gnome.h>\n\n");
+
+      if (glade_project_get_gettext_support (data->project))
+	{
+	  /* bonobo-i18n.h doesn't include the Q_ macro so we add it here. */
+	  fprintf (fp,
+		   "#undef Q_\n"
+		   "#ifdef ENABLE_NLS\n"
+		   "#  define Q_(String) g_strip_context ((String), gettext (String))\n"
+		   "#else\n"
+		   "#  define Q_(String) g_strip_context ((String), (String))\n"
+		   "#endif\n\n\n");
+	}
+    }
   else
     {
       fprintf (fp,
@@ -1923,6 +1943,7 @@ source_write_support_files (GbWidgetWriteSourceData * data)
 		   "#  include <libintl.h>\n"
 		   "#  undef _\n"
 		   "#  define _(String) dgettext (PACKAGE, String)\n"
+		   "#  define Q_(String) g_strip_context ((String), gettext (String))\n"
 		   "#  ifdef gettext_noop\n"
 		   "#    define N_(String) gettext_noop (String)\n"
 		   "#  else\n"
@@ -1935,6 +1956,7 @@ source_write_support_files (GbWidgetWriteSourceData * data)
 		   "#  define dcgettext(Domain,Message,Type) (Message)\n"
 		   "#  define bindtextdomain(Domain,Directory) (Domain)\n"
 		   "#  define _(String) (String)\n"
+		   "#  define Q_(String) g_strip_context ((String), (String))\n"
 		   "#  define N_(String) (String)\n"
 		   "#endif\n\n\n");
 	}
@@ -2379,6 +2401,41 @@ source_add (GbWidgetWriteSourceData * data, const gchar * fmt, ...)
 }
 
 
+void
+source_add_translator_comments (GbWidgetWriteSourceData *data,
+				gboolean translatable,
+				const gchar *comments)
+{
+  /* If the property isn't being translated we don't bother outputting the
+     translator comments. */
+  if (!translatable || !comments || comments[0] == '\0')
+    return;
+
+  /* We simply output it in a C comment.
+     FIXME: If the comments contain an end of comment marker it won't
+     compile. */
+  source_add (data, "  /* %s */\n", comments);
+}
+
+
+void
+source_add_translator_comments_to_buffer (GbWidgetWriteSourceData *data,
+					  GladeSourceBuffer buffer,
+					  gboolean translatable,
+					  const gchar *comments)
+{
+  /* If the property isn't being translated we don't bother outputting the
+     translator comments. */
+  if (!translatable || !comments || comments[0] == '\0')
+    return;
+
+  /* We simply output it in a C comment.
+     FIXME: If the comments contain an end of comment marker it won't
+     compile. */
+  source_add_to_buffer (data, buffer, "  /* %s */\n", comments);
+}
+
+
 void
 source_add_decl (GbWidgetWriteSourceData * data, const gchar * fmt, ...)
 {
@@ -2466,7 +2523,16 @@ gchar *
 source_make_string (const gchar * text,
 		    gboolean translatable)
 {
-  return source_make_string_internal (text, translatable, FALSE);
+  return source_make_string_internal (text, translatable, FALSE, FALSE);
+}
+
+
+gchar *
+source_make_string_full (const gchar * text,
+			 gboolean translatable,
+			 gboolean context)
+{
+  return source_make_string_internal (text, translatable, FALSE, context);
 }
 
 
@@ -2474,7 +2540,7 @@ gchar *
 source_make_static_string (const gchar * text,
 			   gboolean translatable)
 {
-  return source_make_string_internal (text, translatable, TRUE);
+  return source_make_string_internal (text, translatable, TRUE, FALSE);
 }
 
 
@@ -2487,7 +2553,8 @@ source_make_static_string (const gchar * text,
 static gchar *
 source_make_string_internal (const gchar * text,
 			     gboolean translatable,
-			     gboolean is_static)
+			     gboolean is_static,
+			     gboolean context)
 {
   static GString *buffer = NULL;
 
@@ -2510,7 +2577,14 @@ source_make_string_internal (const gchar * text,
   /* Output the C code to start the string. */
   if (translatable)
     {
-      g_string_append (buffer, is_static ? "N_(\"" : "_(\"");
+      /* Static "N_" overrides any context setting, though they shouldn't
+	 really both be passed in as TRUE anyway. */
+      if (is_static)
+	g_string_append (buffer, "N_(\"");
+      else if (context)
+	g_string_append (buffer, "Q_(\"");
+      else
+	g_string_append (buffer, "_(\"");
     }
   else
     {
